@@ -9,8 +9,7 @@ import six
 
 from six.moves.urllib.parse import urlparse, parse_qs
 
-from OpenSSL import SSL
-
+import flextls
 
 from sslscan.__about__ import (
     __author__, __copyright__, __email__, __license__, __summary__, __title__,
@@ -113,27 +112,38 @@ class Scanner(object):
         module.config.set_values(config)
         self.append(module)
 
-    def get_enabled_methods(self):
+    def get_enabled_versions(self):
         """
         Uses the scanner config to create and return a list of all enabled
-        SSL methods.
+        SSL/TLS protocol versions.
 
         :return: List of methods
         :rtype: List
         """
 
-        methods = []
+        versions = []
         if self.config.get_value('ssl2'):
-            methods.append(SSL.SSLv2_METHOD)
+            versions.append(flextls.registry.version.SSLv2)
         if self.config.get_value('ssl3'):
-            methods.append(SSL.SSLv3_METHOD)
+            versions.append(flextls.registry.version.SSLv3)
         if self.config.get_value('tls10'):
-            methods.append(SSL.TLSv1_METHOD)
+            versions.append(flextls.registry.version.TLSv10)
         if self.config.get_value('tls11'):
-            methods.append(SSL.TLSv1_1_METHOD)
+            versions.append(flextls.registry.version.TLSv11)
         if self.config.get_value('tls12'):
-            methods.append(SSL.TLSv1_2_METHOD)
-        return methods
+            versions.append(flextls.registry.version.TLSv12)
+
+        return versions
+
+    def get_handler(self):
+        """
+        Get the active protocol handler.
+
+        :return: Instance of the handler
+        :rtype: sslscan.module.handler.BaseHandler
+        """
+
+        return self.handler
 
     def get_knowledge_base(self):
         """Return the knowledge base used by this scanner."""
@@ -154,10 +164,11 @@ class Scanner(object):
         """
 
         logger.debug("Loading handler from URI: %s", host_uri)
-        if not re.search('^([a-z]+:)?\/\/', host_uri):
+        if not re.search('^([a-zA-Z0-9]+:)?\/\/', host_uri):
             host_uri = '//' + host_uri
         uri = urlparse(host_uri)
         name = uri.scheme
+        name = name.lower()
         if name == '':
             name = 'tcp'
         module = self._module_manager.get(name, base_class=BaseHandler)
@@ -185,6 +196,11 @@ class Scanner(object):
             return NoneRating()
         return module()
 
+    def reset_knowledge_base(self):
+        """Create and activate a new knowledge base for this scanner."""
+
+        self._kb = KnowledgeBase()
+
     def run(self):
         """Execute all scan and report modules attached to the scanner."""
         self.run_scans()
