@@ -20,12 +20,6 @@ class HTTP(TCP):
         self.port = 443
         TCP.__init__(self, **kwargs)
 
-    def connect(self):
-        conn = socket()
-        conn.connect((self.host, self.port))
-
-        return conn
-
     def request(self, conn):
         conn.send(b"GET / HTTP/1.1\r\n")
         conn.send(b"User-Agent: SSLScan\r\n")
@@ -59,7 +53,8 @@ class HTTP(TCP):
             name, sep, value = line.partition(":")
             result["headers"].append((name, value))
 
-        return result
+        if self._server_info is None:
+            self._server_info = result
 
 
 modules.register(HTTP)
