@@ -4,6 +4,7 @@ from sslscan.module import BaseModule
 class BaseRating(BaseModule):
     def __init__(self, **kwargs):
         BaseModule.__init__(self, **kwargs)
+        self._rules = {}
 
     def rate(self, rule_id, data):
         rules = self._rules.get(rule_id)
@@ -19,4 +20,6 @@ class BaseRating(BaseModule):
 class NoneRating(BaseRating):
     name="none"
 
-    _rules = {}
+    def __init__(self, **kwargs):
+        BaseModule.__init__(self, **kwargs)
+        self._rules = {}
