@@ -23,9 +23,9 @@ class Terminal(BaseReport):
 
         print("Supported Client Cipher(s):")
         for cipher in ciphers:
-            rating_bits = self._rating.rate('cipher.bits', cipher)
-            rating_method = self._rating.rate('cipher.method', cipher)
-            rating_name = self._rating.rate('cipher.name', cipher)
+            rating_bits = self._rating.rate('cipher.bits', cipher.bits)
+            rating_method = self._rating.rate('cipher.method', cipher.method)
+            rating_name = self._rating.rate('cipher.name', cipher.name)
             print(
                 "  {3}{0:7}{6} {4}{1:>9}{6} {5}{2}{6}".format(
                     cipher.method_name,
@@ -70,58 +70,133 @@ class Terminal(BaseReport):
                 )
             print("")
 
-    def _print_server_certificate(self, kb):
-        x509 = kb.get("server.certificate")
-        if x509 is None:
+    def _print_server_certificate_chain(self, kb):
+        cert_chain = kb.get("server.certificate_chain")
+        if cert_chain is None:
             return
+        cert_chain.reverse()
+        for i, x509 in enumerate(cert_chain):
+            print(
+                "SSL Certificate({0}/{1}):".format(
+                    i + 1,
+                    len(cert_chain)
+                )
+            )
+            self._print_server_certificate_x509(x509)
+
+    def _print_server_certificate_x509(self, x509):
 
         x509name_members = [
-            "countryName",
-            "stateOrProvinceName",
-            "localityName",
-            "organizationName",
-            "organizationalUnitName",
-            "commonName",
-            "emailAddress"
+            ("country_name", "countryName"),
+            ("state_or_province_name", "stateOrProvinceName"),
+            ("locality_name", "localityName"),
+            ("organization_name", "organizationName"),
+            ("organizational_unit_name", "organizationalUnitName"),
+            ("common_name", "commonName"),
+            ("email_address", "emailAddress")
         ]
-        print("SSL Certificate:")
 #        print("    Certificate blob:")
 #        print(x509.get_certificate_blob())
-        print("  Version: %lu" % x509.get_version())
-        tmp = x509.get_serial_number()
-        print("  Serial Number: %lu (0x%lx)" % (tmp, tmp))
-        print("  Signature Algorithm: %s" % x509.get_signature_algorithm().decode("ASCII"))
+
+        version = x509.get_version()
+        rating_version = self._rating.rate('server.certificate.x509.version', version)
+        print(
+            "  Version: {1}{0}{2}".format(
+                version,
+                helper.rating2color(self.color, rating_version),
+                self.color.RESET
+            )
+        )
+
+        serial = x509.get_serial_number()
+        rating_serial = self._rating.rate('server.certificate.x509.serial_number', serial)
+        print(
+            "  Serial Number: {1}{0} (0x{0:x}){2}".format(
+                serial,
+                helper.rating2color(self.color, rating_serial),
+                self.color.RESET
+            )
+        )
+
+        signature_algorithm = x509.get_signature_algorithm().decode("ASCII")
+        rating_signature_algorithm = self._rating.rate(
+            "server.certificate.x509.signature_algorithm",
+            signature_algorithm
+        )
+        print(
+            "  Signature Algorithm: {1}{0}{2}".format(
+                signature_algorithm,
+                helper.rating2color(self.color, rating_signature_algorithm),
+                self.color.RESET
+            )
+        )
+
         tmp_issuer = x509.get_issuer()
         if tmp_issuer:
             print("  Issuer:")
-            for tmp_name in x509name_members:
+            for rating_name, tmp_name in x509name_members:
                 tmp_value = getattr(tmp_issuer, tmp_name)
+                rating_tmp = self._rating.rate(
+                    "server.certificate.x509.{}".format(rating_name),
+                    tmp_value
+                )
                 if tmp_value is None:
                     tmp_value = ""
-                print("    {}: {}".format(
+                print("    {0}: {2}{1}{3}".format(
                     tmp_name,
-                    tmp_value
-                ))
+                    tmp_value,
+                    helper.rating2color(self.color, rating_tmp),
+                    self.color.RESET
+                )
+)
         tmp_date = datetime.datetime.strptime(
             x509.get_notBefore().decode("ASCII"),
             "%Y%m%d%H%M%SZ"
         )
-        print("  Not valid before: %s" % str(tmp_date))
+        rating_date = self._rating.rate(
+            "server.certificate.x509.not_before",
+            tmp_date
+        )
+        print(
+            "  Not valid before: {1}{0}{2}".format(
+                str(tmp_date),
+                helper.rating2color(self.color, rating_date),
+                self.color.RESET
+            )
+        )
+
         tmp_date = datetime.datetime.strptime(
             x509.get_notAfter().decode("ASCII"),
             "%Y%m%d%H%M%SZ"
         )
-        print("  Not valid after: %s" % str(tmp_date))
+        rating_date = self._rating.rate(
+            "server.certificate.x509.not_after",
+            tmp_date
+        )
+        print(
+            "  Not valid after: {1}{0}{2}".format(
+                str(tmp_date),
+                helper.rating2color(self.color, rating_date),
+                self.color.RESET
+            )
+        )
+
         tmp_subject = x509.get_subject()
         if tmp_subject:
             print("  Subject:")
-            for tmp_name in x509name_members:
+            for rating_name, tmp_name in x509name_members:
                 tmp_value = getattr(tmp_subject, tmp_name)
+                rating_tmp = self._rating.rate(
+                    "server.certificate.x509.{}".format(rating_name),
+                    tmp_value
+                )
                 if tmp_value is None:
                     tmp_value = ""
-                print("      {}: {}".format(
+                print("    {0}: {2}{1}{3}".format(
                     tmp_name,
-                    tmp_value
+                    tmp_value,
+                    helper.rating2color(self.color, rating_tmp),
+                    self.color.RESET
                 ))
 #        pk = x509.get_pubkey()
 #        if pk:
@@ -169,17 +244,20 @@ class Terminal(BaseReport):
 
         print("Supported Server Cipher(s):")
         for cipher in ciphers:
-            rating_bits = self._rating.rate('cipher.bits', cipher)
-            rating_method = self._rating.rate('cipher.method', cipher)
-            rating_name = self._rating.rate('cipher.name', cipher)
+            rating_bits = self._rating.rate('cipher.bits', cipher.cipher_suite.bits)
+            rating_protocol_version = self._rating.rate(
+                'cipher.protocol_version',
+                cipher.protocol_version
+            )
+            rating_name = self._rating.rate('cipher.name', cipher.cipher_suite.name)
             print(
                 "  {0:9} {5}{1:7}{8} {6}{2:>9}{8} {7}{3}{8}  {4}".format(
                     cipher.status_name.capitalize(),
-                    cipher.method_name,
-                    "%d bits" % cipher.bits,
-                    cipher.name,
+                    cipher.protocol_version_name,
+                    "%d bits" % cipher.cipher_suite.bits,
+                    cipher.cipher_suite.name,
                     "", # reserved for alert info
-                    helper.rating2color(self.color, rating_method),
+                    helper.rating2color(self.color, rating_protocol_version),
                     helper.rating2color(self.color, rating_bits),
                     helper.rating2color(self.color, rating_name),
                     self.color.RESET
@@ -192,24 +270,28 @@ class Terminal(BaseReport):
             return
 
         reneg_support = kb.get("server.renegotiation.support")
-        rating_renegotiation = self._rating.rate("renegotiation.support", reneg_support)
-        rating_color = helper.rating2color(self.color, rating_renegotiation)
+        rating_renegotiation = self._rating.rate(
+            "server.renegotiation.support",
+            reneg_support
+        )
         print("TLS renegotiation:")
         print(
             "  Supported: {1}{0}{2}".format(
                 "yes" if reneg_support else "no",
-                rating_color,
+                helper.rating2color(self.color, rating_renegotiation),
                 self.color.RESET
             )
         )
 
         reneg_secure = kb.get("server.renegotiation.secure")
-        rating_renegotiation = self._rating.rate("renegotiation.secure", reneg_secure)
-        rating_color = helper.rating2color(self.color, rating_renegotiation)
+        rating_renegotiation = self._rating.rate(
+            "server.renegotiation.secure",
+            reneg_secure
+        )
         print(
             "  Secure: {1}{0}{2}".format(
                 "yes" if reneg_secure else "no",
-                rating_color,
+                helper.rating2color(self.color, rating_renegotiation),
                 self.color.RESET
             )
         )
@@ -222,16 +304,36 @@ class Terminal(BaseReport):
 
         print("Preferred Server Cipher(s):")
         for cipher in ciphers:
-            rating_bits = self._rating.rate('cipher.bits', cipher)
-            rating_method = self._rating.rate('cipher.method', cipher)
-            rating_name = self._rating.rate('cipher.name', cipher)
+            rating_version = self._rating.rate('cipher.version', cipher.protocol_version)
+            if cipher.cipher_suite == None:
+                print(
+                    "  {1}{0:7}{2} Protocol version not supported".format(
+                        cipher.protocol_version_name,
+                        helper.rating2color(self.color, rating_version),
+                        self.color.RESET
+                    )
+                )
+                continue
+
+            if cipher.cipher_suite == False:
+                print(
+                    "  {1}{0:7}{2} No preferred cipher suite".format(
+                        cipher.protocol_version_name,
+                        helper.rating2color(self.color, rating_version),
+                        self.color.RESET
+                    )
+                )
+                continue
+
+            rating_bits = self._rating.rate('cipher.bits', cipher.cipher_suite.bits)
+            rating_name = self._rating.rate('cipher.name', cipher.cipher_suite.name)
             print(
                 "  {4}{0:7}{7} {5}{1:>9}{7} {6}{2}{7} {3}".format(
-                    cipher.method_name,
-                    "%d bits" % cipher.bits,
-                    cipher.name,
+                    cipher.protocol_version_name,
+                    "%d bits" % cipher.cipher_suite.bits,
+                    cipher.cipher_suite.name,
                     "", # alert info
-                    helper.rating2color(self.color, rating_method),
+                    helper.rating2color(self.color, rating_version),
                     helper.rating2color(self.color, rating_bits),
                     helper.rating2color(self.color, rating_name),
                     self.color.RESET
@@ -245,21 +347,49 @@ class Terminal(BaseReport):
 
         print("Session:")
         compression = kb.get("server.session.compression")
+        rating_compression = self._rating.rate(
+            "server.session.compression",
+            compression
+        )
         if compression is not None:
             if not compression:
-                compression = "None"
-            print("  Compression: {0}".format(compression))
+                compression_name = "None"
+            else:
+                compression_name = compression.name
+            print(
+                "  Compression: {1}{0}{2}".format(
+                    compression_name,
+                    helper.rating2color(self.color, rating_compression),
+                    self.color.RESET
+                )
+            )
 
         expansion = kb.get("server.session.expansion")
+        rating_expansion = self._rating.rate(
+            "server.session.expansion",
+            expansion
+        )
         if expansion is not None:
             if not expansion:
                 expansion = "None"
-            print("  Expansion: {0}".format(expansion))
+            print(
+                "  Expansion: {0}".format(
+                    expansion,
+                    helper.rating2color(self.color, rating_expansion),
+                    self.color.RESET
+                )
+            )
 
         print("")
 
     def run(self):
         kb = self._scanner.get_knowledge_base()
+        handler = self._scanner.get_handler()
+
+        print("")
+        print(handler.hostname)
+        print("="*len(handler.hostname))
+        print("")
 
         self._rating = self._scanner.load_rating(self.config.get_value("rating"))
 
@@ -269,7 +399,7 @@ class Terminal(BaseReport):
 
         self._print_server_preferred_ciphers(kb)
 
-        self._print_server_certificate(kb)
+        self._print_server_certificate_chain(kb)
 
         self._print_server_session(kb)
 
